(******************************************************************************
 * AcDbf                                                                      *
 ******************************************************************************)
unit AcDbf;

(******************************************************************************
 * interface                                                                  *
 ******************************************************************************)
 interface

(******************************************************************************
 * uses                                                                       *
 ******************************************************************************)
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs;


(******************************************************************************
 * AC_DBF_HEADER_STRUTT                                                       *
 ******************************************************************************)
 type AC_DBF_HEADER_STRUTT = record
   versione,
   Anno,
   mese,
   Giorno             : Byte;
   Numero_Record      : Integer; // 32 bit
   lunghezza_struttura,
   lunghezza_record   : Word; // 16 bit
   riservati          : Array[1..20] of byte;
 end;

 AC_DBF_FIELD_STRUTT = Record
    nome_campo : Array[0..10] of char;
    tipo_campo : Char; // tipo del campo
                       // C N L D M
    address    : Integer; // 32 Bit
    dimensione : Byte;
    decimali   : Byte;
    riservati  : Array[1..14] of byte;
 End;

 PAC_DBF_FIELD_STRUTT_IN = ^AC_DBF_FIELD_STRUTT_IN;
 AC_DBF_FIELD_STRUTT_IN = Record // Questa struttura  a uso interno
    Campo   : AC_DBF_FIELD_STRUTT;
    Next    : PAC_DBF_FIELD_STRUTT_IN;
 End;

 TACDBF_Procedure = Procedure of object;
 PAC_DB_Componenti = ^AC_DB_Componenti;
 AC_DB_Componenti = record
   Proced   : TACDBF_Procedure;
   Next     : PAC_DB_Componenti;
 end;

(******************************************************************************
 * ACDbf                                                                      *
 ******************************************************************************)
type
  TACDbf = class(TComponent)
  private
    { Private declarations }
    F_FileName  : String;
    F_Active    : Boolean;
    H_File      : Integer;  // Handle del DBF
    F_Versione  : Integer;
    dbf_Str     : AC_DBF_HEADER_STRUTT;
    Dbf_Field   : PAC_DBF_FIELD_STRUTT_IN;
    F_FieldName : TStringList; // Elenco dei campi
    List_Comp   : PAC_DB_Componenti; // Elenco componenti collegati
    Procedure Close_File;
    Procedure Load_Fields;
  protected
    { Protected declarations }
    Procedure SetActive(fl : Boolean);
    Procedure SetFileName(Txt:String);
    Procedure Set_FieldName(T:TStringList);
    Procedure Read_Rec;
    procedure Get_Estremi_Field(Nome:String;var v1,v2:Integer);
    procedure Aggiorna_Componenti;
  public
    { Public declarations }
    My_Rec             : PChar;
    constructor Create(AOwner: TComponent); Override;
    destructor Destroy; Override;
    Function Get_Field_Str(Nome:String):String;
    Procedure MoveFirst;
    Procedure MoveLast;
    Procedure MoveNext;
    Procedure MovePrevious;
    Procedure Add_Control(pr:TACDBF_Procedure);
    Procedure Remove_Control(pr:TACDBF_Procedure);

    Procedure Libera_Componenti;

  published
    { Published declarations }
    Property Active    : Boolean Read F_Active Write SetActive;
    property FileName  : String Read F_FileName Write SetFileName;
    Property Version   : Integer Read F_Versione Write F_Versione;
    Property FieldName : TStringList Read F_FieldName Write Set_FieldName;

  end;

(******************************************************************************
 * implementation                                                             *
 ******************************************************************************)
 implementation

(******************************************************************************
 * Set_FieldName                                                              *
 ******************************************************************************)
Procedure TACDbf.Set_FieldName(T:TStringList);
Begin
End;

(******************************************************************************
 * Load_Fields                                                                *
 ******************************************************************************)
Procedure TACDbf.Load_Fields;
Var
  App_Dbf_Field1,
  App_Dbf_Field2 : PAC_DBF_FIELD_STRUTT_IN;
  Field          : AC_DBF_FIELD_STRUTT;
  Numero_Campi   : Integer;
  C1             : Integer;
Begin
  F_FieldName.Clear;
  Numero_Campi := (dbf_Str.Lunghezza_Struttura - 32) div 32;
  For c1 := 1 to Numero_Campi do
  Begin
    FileRead(H_File,Field,sizeof(AC_DBF_FIELD_STRUTT));
    New (App_Dbf_Field1);
    App_Dbf_Field1^.Campo := Field;
    App_Dbf_Field1^.Next  := Nil;
    F_FieldName.Add (Field.nome_campo);
    if(Dbf_Field=Nil) then Dbf_Field:=App_Dbf_Field1
    Else
    Begin
      App_Dbf_Field2 := Dbf_Field;
      While(App_Dbf_Field2^.Next<>Nil) Do App_Dbf_Field2:=App_Dbf_Field2^.Next;
      App_Dbf_Field2^.Next := App_Dbf_Field1;
    End;

  End;

End;
(******************************************************************************
 * Close_File                                                                 *
 ******************************************************************************)
Procedure TACDbf.Close_File;
Var
  App_Dbf_Field  : PAC_DBF_FIELD_STRUTT_IN;
Begin
  if (Active=True) then
  begin
    FileClose(H_File);
    StrDispose(My_Rec);
  end;
  F_Active:=False;
  While(Dbf_Field<>Nil) Do
  Begin
    App_Dbf_Field := Dbf_Field^.Next;
    Dispose(Dbf_Field);
    Dbf_Field := App_Dbf_Field;
  End;
  F_FieldName.Clear;
End;

(******************************************************************************
 * SetActive                                                                  *
 ******************************************************************************)
Procedure TACDbf.SetActive(fl : Boolean);
Begin
 if (Fl=True) then
 begin
   Close_File;
   if (FileExists(FileName)=True) then
   Begin
     F_Active := False;
     H_File := FileOpen(FileName,fmOpenRead);
     if(H_File>0) then
     begin
       FileRead(H_File,Dbf_Str,sizeof(AC_DBF_HEADER_STRUTT));
       My_Rec := StrAlloc(DBf_Str.lunghezza_record+1);
       Version := Dbf_Str.Versione;
       F_Active := True;
       Load_Fields;
       MoveFirst;
     end;
   end;
 End
 else
 Begin
   Close_File;
 End;
End;


(******************************************************************************
 * constructor                                                                *
 ******************************************************************************)
constructor TACDbf.Create(AOwner: TComponent);
Begin
  Inherited;
  F_FieldName := TStringList.Create;
  FileName    := '';
  Active      := False;
  Version     := 0;
  List_Comp   := Nil;

End;


(******************************************************************************
 * destructor                                                                 *
 ******************************************************************************)
destructor TACDbf.Destroy;
Begin
  Active := False;
  F_FieldName.Free;
  Libera_Componenti;
  Inherited;
End;

(******************************************************************************
 * Read_Rec                                                                   *
 ******************************************************************************)
Procedure TACDbf.Read_Rec;
//var txt : string[200];
Begin
  FileRead(H_File,My_Rec^,DBf_Str.lunghezza_record);
End;

(******************************************************************************
 * MoveFirst                                                                  *
 ******************************************************************************)
Procedure TACDbf.MoveFirst;
Begin
  if(Active=True) Then
  Begin
    FileSeek(H_File,dbf_Str.lunghezza_struttura,0);
    Read_Rec;
    FileSeek(H_File,dbf_Str.lunghezza_struttura,0);
    Aggiorna_Componenti;
  End;
End;
(******************************************************************************
 * MoveLast                                                                   *
 ******************************************************************************)
Procedure TACDbf.MoveLast;
Var File_EOF : byte;
Begin
  if(Active=True) Then
  Begin
    FileSeek(H_File,-1,2);
    FileRead(H_File,File_EOF,1);
    if(File_EOF=26) then FileSeek(H_File,-1,2)
    else FileSeek(H_File,0,2);
    FileSeek(H_File,-dbf_Str.lunghezza_record,1);
    Aggiorna_Componenti;
 End;
End;
(******************************************************************************
 * MoveNext                                                                   *
 ******************************************************************************)
Procedure TACDbf.MoveNext;
Begin
  if(Active=True) Then
  Begin
    FileSeek(H_File,dbf_Str.lunghezza_record,1);
    Read_Rec;
    Aggiorna_Componenti;
  End;
End;
(******************************************************************************
 * MovePrevious                                                               *
 ******************************************************************************)
Procedure TACDbf.MovePrevious;
Begin
  if(Active=True) Then
  Begin
    if (FileSeek(H_File,-dbf_Str.lunghezza_record,1)<dbf_Str.lunghezza_struttura) then
    Begin
       FileSeek(H_File,dbf_Str.lunghezza_struttura,0);
    End;
    Read_Rec;
    FileSeek(H_File,-dbf_Str.lunghezza_record,1);
    Aggiorna_Componenti;
  End;
End;

(******************************************************************************
 * destructor                                                                 *
 ******************************************************************************)
Procedure TACDbf.SetFileName(Txt:String);
Begin
  if(Not Active) Then
  Begin
    F_FileName:=Txt;
  End;
End;

(******************************************************************************
 * Get_Estermi_Field                                                          *
 ******************************************************************************)
procedure TACDbf.Get_Estremi_Field(Nome:String;var v1,v2:Integer);
Var
 Field   : PAC_DBF_FIELD_STRUTT_IN;
begin
 Field := Dbf_Field;
 v1:=1;
 Nome:=UpperCase(Nome);
 While(Field<>Nil)do
 begin
   if(UpperCase(Field.campo.Nome_Campo)=Nome) then
   begin
     v2 := v1 + Field.campo.dimensione-1;
     Inc(v1);
     inc(v2);
     exit;
   end;
   v1 := V1 + Field.campo.dimensione;
   Field:=Field^.Next;
 end;
 v1:=0;
 v2:=0;
end;

(******************************************************************************
 * Get_Field_Str                                                              *
 ******************************************************************************)
Function TACDbf.Get_Field_Str(Nome:String):String;
Var v1,v2:Integer;
Begin
  Get_Estremi_Field(Nome,v1,v2);
  if(v2<>0) then Get_Field_Str:= Copy(My_rec,v1,v2-v1+1)
  else Get_Field_Str := '';
End;

(******************************************************************************
 * Add_Control                                                                *
 ******************************************************************************)
Procedure TACDbf.Add_Control(pr:TACDBF_Procedure);


Var
  L1   : PAC_DB_Componenti; //
  L2   : PAC_DB_Componenti; //
Begin
  New(L1);
  //L1^.Comp := TComponent.Create(Self);
  L1^.Proced := Pr;
  L1^.Next := Nil;
  if(List_Comp=Nil) then
  begin
    List_Comp := L1;
  End
  Else
  Begin
    L2 := List_Comp;
    While(L2^.Next<>Nil) Do L2:=L2^.Next;
    L2^.Next := L1;
  End;
End;

(******************************************************************************
 * Remove_Win                                                                 *
 ******************************************************************************)
Procedure TACDbf.Remove_Control(pr:TACDBF_Procedure);

Var
  Corrente    : PAC_DB_Componenti; //
  Precedente  : PAC_DB_Componenti; //
  p1,
  p2          : TACDBF_Procedure;

Begin
  if(List_Comp=Nil) then
  begin
    Exit;
  End
  Else
  Begin
    Corrente := List_Comp;
    Precedente := Nil;
    While(Corrente<>Nil) Do
    Begin
      //pr := Corrente^.Proced;
      p1 := Corrente^.Proced;
      p2 := pr;
(*$t-*)
      if(@p1=@p2) then
      begin
        //L1^.Comp.Free;
        if(Precedente=Nil) then
        begin
          List_Comp := Corrente^.Next;
          Dispose(Corrente);
          Exit;
        end
        Else
        Begin
          Precedente^.Next := Corrente^.Next;
          Dispose(Corrente);
          Exit;
        End;
      End;
      Precedente := Corrente;
      Corrente:=Corrente^.Next;
    End;
    Precedente^.Next := List_Comp;
  End;
End;

(******************************************************************************
 * Libera_Componenti                                                           *
 ******************************************************************************)
Procedure TACDbf.Libera_Componenti;
Var
  Corrente    : PAC_DB_Componenti; //
  Successivo  : PAC_DB_Componenti; //

Begin
  if(List_Comp<>Nil) then
  Begin
    Corrente := List_Comp;
    While (Corrente<>Nil) do
    begin
      Successivo := Corrente^.Next;
      Dispose(Corrente);
      Corrente:=Successivo;
    end;
  End;


End;

(******************************************************************************
 * Aggiorna_componenti                                                         *
 ******************************************************************************)
procedure TACDbf.Aggiorna_Componenti;
Var
  Corrente    : PAC_DB_Componenti; //
Begin
  Corrente := List_Comp;
  While (Corrente<>Nil) do
  begin
    Corrente.Proced;
    Corrente:=Corrente^.Next;
  End;
End;
end.
